#include "dropdownmenu.h"
#include <QWidgetAction>
#include <QStyleOption>
#include <QApplication>
#include <QPainter>
#include <QDebug>
#include <QGSettings>

MenuItem::MenuItem(QString week, int id, QWidget *parent) : QWidget(parent)
{
    m_id = id;
    m_pweekLabel = new QLabel();
    m_pweekLabel->setText(week);
    m_pselectedLabelIcon = new QLabel();
    m_pselectedLabelIcon->setFixedSize(16, 16);
    QIcon labelIcon = QIcon::fromTheme("object-select-symbolic");
    m_pselectedLabelIcon->setPixmap(labelIcon.pixmap(QSize(16, 16)));
    m_pselectedLabelIcon->setProperty("useIconHighlightEffect", 0x8);
    m_pHWeekLayout = new QHBoxLayout();
    m_pHWeekLayout->setContentsMargins(17, 0, 0, 0);
    m_pHWeekLayout->addWidget(m_pselectedLabelIcon);
    m_pHWeekLayout->addWidget(m_pweekLabel);
    QSizePolicy policy = m_pselectedLabelIcon->sizePolicy();
    policy.setRetainSizeWhenHidden(true);
    m_pselectedLabelIcon->setSizePolicy(policy);
    m_pselectedLabelIcon->setVisible(false);
    this->setLayout(m_pHWeekLayout);
    this->setFixedSize(306, 35);
}

//显示或取消勾选标志
void MenuItem::mouseReleaseEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        if (!this->m_pselectedLabelIcon->isVisible()) {
            m_pselectedLabelIcon->setVisible(true);
            emit showLabelIcon(m_id);
        } else {
            m_pselectedLabelIcon->setVisible(false);
            emit hideLabelIcon(m_id);
        }
    }
    QWidget::mouseReleaseEvent(event);
}

void MenuItem::enterEvent(QEvent *e)
{
    Q_UNUSED(e);
    m_mouseEnter = true;
    repaint();
}

void MenuItem::leaveEvent(QEvent *e)
{
    Q_UNUSED(e);
    m_mouseEnter = false;
    repaint();
}

void MenuItem::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    opt.rect.adjust(0,0,0,0);
    if (m_mouseEnter) {
        QPalette palette;
        palette.setColor(QPalette::Active, QPalette::WindowText,Qt::white);
        m_pweekLabel->setPalette(palette);
        p.setBrush(opt.palette.color(QPalette::Link));
    } else {
        if (QGSettings::isSchemaInstalled(QString("org.ukui.style").toLocal8Bit())){
            QGSettings gsetting(QString("org.ukui.style").toLocal8Bit());
            if (gsetting.keys().contains("styleName")){
                QPalette palette;
                if (gsetting.get("styleName").toString() == "ukui-default") {
                    palette.setColor(QPalette::Active, QPalette::WindowText,Qt::black);
                } else {
                    palette.setColor(QPalette::Active, QPalette::WindowText,Qt::white);
                }
                m_pweekLabel->setPalette(palette);
            }
        }
        p.setBrush(opt.palette.color(QPalette::ToolTipBase));
    }
    p.setRenderHint(QPainter::HighQualityAntialiasing);
    p.setOpacity(1);
    p.setPen(Qt::NoPen);
    p.drawRoundedRect(opt.rect, 6, 6);
}

DropDownMenu::DropDownMenu()
{
    initWeekDateList();
    initAllDateWidget();
}

/* 初始化链表， 将所有的选项保存到链表中 */
void DropDownMenu::initWeekDateList()
{
    m_pWeekDateList.append(QObject::tr("never"));
    m_pWeekDateList.append(QObject::tr("Only this shutdown"));
    m_pWeekDateList.append(QObject::tr("Everyday"));
    m_pWeekDateList.append(QObject::tr("Mon"));
    m_pWeekDateList.append(QObject::tr("Tue"));
    m_pWeekDateList.append(QObject::tr("Wed"));
    m_pWeekDateList.append(QObject::tr("Thu"));
    m_pWeekDateList.append(QObject::tr("Fri"));
    m_pWeekDateList.append(QObject::tr("Sat"));
    m_pWeekDateList.append(QObject::tr("Sun"));
    return;
}

void DropDownMenu::initAllDateWidget()
{
    this->setFixedWidth(324);
    m_selectWeek.clear();
    for (int i = 0; i < m_pWeekDateList.count(); i++) {
        MenuItem *item = new MenuItem(m_pWeekDateList[i], i, this);
        connect(item, SIGNAL(showLabelIcon(int)), this, SLOT(handleShowLabelSlot(int)));
        connect(item, SIGNAL(hideLabelIcon(int)), this, SLOT(handleHideLabelSlot(int)));
        QWidgetAction *action = new QWidgetAction(this);
        action->setDefaultWidget(item);
        this->addAction(action);
    }
}

void DropDownMenu::initLabelStatus(QList<int> selectWeek)
{
    m_selectWeek.clear();
    for(int i = 0; i < selectWeek.count(); i++) {
        QWidgetAction *wAction = dynamic_cast<QWidgetAction *>(this->actions().at(selectWeek[i]));
        MenuItem *item = dynamic_cast<MenuItem *>(wAction->defaultWidget());
        item->m_pselectedLabelIcon->setVisible(true);
        m_selectWeek.append(selectWeek[i]);
    }
}

//返回列表选中的组合
QString DropDownMenu::getChosenList(QList<int> selectList)
{
    QString chooseList = m_pWeekDateList[selectList[0]];
    for (int i = 1; i < selectList.count(); i++) {
        chooseList += " " + m_pWeekDateList[selectList[i]];
    }
    return chooseList;
}

//从gsetting中获取选取星期值
QList<int> DropDownMenu::getWeekDateList(QString indexString)
{
    QStringList indexList = indexString.split(" ");
    QList<int> selectWeek;
    selectWeek.clear();
    for (int i = 0; i < indexList.count(); i++) {
        selectWeek.append(indexList.at(i).toInt());
    }
    return selectWeek;
}

void DropDownMenu::handleShowLabelSlot(int id)
{
    if (id <= 2) {
        m_selectWeek.clear();
        for (int i = 0; i < this->actions().count(); ++i) {
            if (i == id) {
                continue;
            }
            QWidgetAction *wAction = dynamic_cast<QWidgetAction *>(this->actions().at(i));
            MenuItem *item = dynamic_cast<MenuItem *>(wAction->defaultWidget());
            item->m_pselectedLabelIcon->setVisible(false);
        }
    } else {
        for (int i = 0; i <= 2; i++) {
            QWidgetAction *wAction = dynamic_cast<QWidgetAction *>(this->actions().at(i));
            MenuItem *item = dynamic_cast<MenuItem *>(wAction->defaultWidget());
            item->m_pselectedLabelIcon->setVisible(false);
            m_selectWeek.removeAll(i);
        }
    }
    m_selectWeek.append(id);
    qSort(m_selectWeek.begin(), m_selectWeek.end());
}

void DropDownMenu::handleHideLabelSlot(int id)
{
    m_selectWeek.removeAll(id);
    m_selectWeek.removeAll(0);
    if (m_selectWeek.isEmpty()) {
        m_selectWeek.append(0);
        QWidgetAction *wAction = dynamic_cast<QWidgetAction *>(this->actions().at(0));
        MenuItem *item = dynamic_cast<MenuItem *>(wAction->defaultWidget());
        item->m_pselectedLabelIcon->setVisible(true);
        this->hide();
    }
}

QStringList DropDownMenu::getWeekStringList()
{
    QStringList weekStringList;
    for (int i = 0; i < m_selectWeek.size(); ++i) {
        weekStringList.append(QString().number(m_selectWeek.at(i)));
    }
    return weekStringList;
}

void DropDownMenu::mouseReleaseEvent(QMouseEvent *event)
{
    QAction *action = this->actionAt(event->pos());
    if (action) {
        for (int i = 0; i < this->actions().count(); i++) {
            if (this->actions().at(i) == action) {
                if (i <= 2 && event->button() == Qt::LeftButton) {
                    QMenu::mouseReleaseEvent(event);
                }
                break;
            }
        }
    } else {
        QMenu::mouseReleaseEvent(event);
    }
}

void DropDownMenu::paintEvent(QPaintEvent *event)
{
    if (QGSettings::isSchemaInstalled(QString("org.ukui.style").toLocal8Bit())){
        QGSettings gsetting(QString("org.ukui.style").toLocal8Bit());
        if (gsetting.keys().contains("styleName")){
            if (gsetting.get("styleName").toString() == "ukui-dark") {
                QStyleOption opt;
                opt.init(this);
                QPainter p(this);
                opt.rect.adjust(0,0,0,0);
                p.setBrush(opt.palette.color(QPalette::ToolTipBase));
                p.setRenderHint(QPainter::HighQualityAntialiasing);
                p.setOpacity(1);
                p.setPen(Qt::NoPen);
                p.drawRoundedRect(opt.rect, 6, 6);
                return;
            }
        }
    }
    QMenu::paintEvent(event);
}
